@@ -3,17 +3,19 @@ Module for integration with BioPython, specifically SeqRecords and SeqFeature
 objects.
 """
 import six
+
 try:
     from Bio.SeqFeature import SeqFeature, FeatureLocation
 except ImportError:
     import warnings
+
     warnings.warn("BioPython must be installed to use this module")
 from .feature import Feature, feature_from_line
 
 _biopython_strand = {
-    '+':  1,
-    '-': -1,
-    '.':  0,
+    "+": 1,
+    "-": -1,
+    ".": 0,
 }
 _feature_strand = dict((v, k) for k, v in _biopython_strand.items())
 
@@ -35,10 +37,10 @@ def to_seqfeature(feature):
         feature = feature_from_line(feature)
 
     qualifiers = {
-        'source': [feature.source],
-        'score': [feature.score],
-        'seqid': [feature.seqid],
-        'frame': [feature.frame],
+        "source": [feature.source],
+        "score": [feature.score],
+        "seqid": [feature.seqid],
+        "frame": [feature.frame],
     }
     qualifiers.update(feature.attributes)
     return SeqFeature(
@@ -48,7 +50,7 @@ def to_seqfeature(feature):
         id=feature.id,
         type=feature.featuretype,
         strand=_biopython_strand[feature.strand],
-        qualifiers=qualifiers
+        qualifiers=qualifiers,
     )
 
 
@@ -60,10 +62,10 @@ def from_seqfeature(s, **kwargs):
     stored as qualifiers.  Any other qualifiers will be assumed to be GFF
     attributes.
     """
-    source = s.qualifiers.get('source', '.')[0]
-    score = s.qualifiers.get('score', '.')[0]
-    seqid = s.qualifiers.get('seqid', '.')[0]
-    frame = s.qualifiers.get('frame', '.')[0]
+    source = s.qualifiers.get("source", ".")[0]
+    score = s.qualifiers.get("score", ".")[0]
+    seqid = s.qualifiers.get("seqid", ".")[0]
+    frame = s.qualifiers.get("frame", ".")[0]
     strand = _feature_strand[s.strand]
 
     # BioPython parses 1-based GenBank positions into 0-based for use within
@@ -73,9 +75,20 @@ def from_seqfeature(s, **kwargs):
     featuretype = s.type
     id = s.id
     attributes = dict(s.qualifiers)
-    attributes.pop('source', '.')
-    attributes.pop('score', '.')
-    attributes.pop('seqid', '.')
-    attributes.pop('frame', '.')
-    return Feature(seqid, source, featuretype, start, stop, score, strand,
-                   frame, attributes, id=id, **kwargs)
+    attributes.pop("source", ".")
+    attributes.pop("score", ".")
+    attributes.pop("seqid", ".")
+    attributes.pop("frame", ".")
+    return Feature(
+        seqid,
+        source,
+        featuretype,
+        start,
+        stop,
+        score,
+        strand,
+        frame,
+        attributes,
+        id=id,
+        **kwargs
+    )
