@@ -11,36 +11,34 @@ important for figuring out how to construct the database.
 import os
 import tempfile
 import itertools
+from contextlib import contextmanager
 from gffutils.feature import feature_from_line
 from gffutils.interface import FeatureDB
 from gffutils import helpers
 from textwrap import dedent
 import six
 from six.moves.urllib.request import urlopen
+
 if six.PY3:
     from urllib import parse as urlparse
 else:
     import urlparse
 
 
-def peek(it, n):
-    _peek = []
-    for _ in range(n):
-        try:
-            _peek.append(six.next(it))
-        except StopIteration:
-            break
-    return _peek, itertools.chain(_peek, it)
-
-
 class Directive(object):
     def __init__(self, line):
         self.info = line
 
 
 class _BaseIterator(object):
-    def __init__(self, data, checklines=10, transform=None,
-                 force_dialect_check=False, dialect=None):
+    def __init__(
+        self,
+        data,
+        checklines=10,
+        transform=None,
+        force_dialect_check=False,
+        dialect=None,
+    ):
         """
         Base class for iterating over features.  In general, you should use
         DataIterator -- so see the docstring of class for argument
@@ -75,29 +73,28 @@ class _BaseIterator(object):
         self.warnings = []
 
         if force_dialect_check and dialect is not None:
-            raise ValueError("force_dialect_check is True, but a dialect "
-                             "is provided")
+            raise ValueError(
+                "force_dialect_check is True, but a dialect " "is provided"
+            )
         if force_dialect_check:
             # In this case, self.dialect remains None.  When
             # parser._split_keyvals gets None as a dialect, it tries to infer
             # a dialect.
             self._iter = self._custom_iter()
         elif dialect is not None:
-            self._observed_dialects = [dialect]
-            self.dialect = helpers._choose_dialect(self._observed_dialects)
-            self._iter = self._custom_iter()
+            self.dialect = dialect
         else:
             # Otherwise, check some lines to determine what the dialect should
             # be
-            self.peek, self._iter = peek(self._custom_iter(), checklines)
-            self._observed_dialects = [i.dialect for i in self.peek]
-            self.dialect = helpers._choose_dialect(self._observed_dialects)
+            _peek = self.peek(checklines)
+            self._peek = _peek
+            self.dialect = helpers._choose_dialect(_peek)
 
     def _custom_iter(self):
         raise NotImplementedError("Must define in subclasses")
 
     def __iter__(self):
-        for i in self._iter:
+        for i in self._custom_iter():
             i.dialect = self.dialect
             if self.transform:
                 i = self.transform(i)
@@ -114,41 +111,55 @@ class _FileIterator(_BaseIterator):
     """
     Subclass for iterating over features provided as a filename
     """
+
+    def peek(self, n):
+        initial = []
+        for i, feature in enumerate(self._custom_iter()):
+            initial.append(feature)
+            if i == n:
+                break
+        return initial
+
     def open_function(self, data):
         data = os.path.expanduser(data)
-        if data.endswith('.gz'):
+        if data.endswith(".gz"):
             import gzip
+
             return gzip.open(data)
         return open(data)
 
     def _custom_iter(self):
+        self.directives = []
         valid_lines = 0
-        for i, line in enumerate(self.open_function(self.data)):
-            if isinstance(line, six.binary_type):
-                line = line.decode('utf-8')
-            line = line.rstrip('\n\r')
-            self.current_item = line
-            self.current_item_number = i
+        with self.open_function(self.data) as fh:
+            for i, line in enumerate(fh):
+                if isinstance(line, six.binary_type):
+                    line = line.decode("utf-8")
+                line = line.rstrip("\n\r")
+                self.current_item = line
+                self.current_item_number = i
 
-            if line == '##FASTA' or line.startswith('>'):
-                return
+                if line == "##FASTA" or line.startswith(">"):
+                    return
 
-            if line.startswith('##'):
-                self._directive_handler(line)
-                continue
+                if line.startswith("##"):
+                    self._directive_handler(line)
+                    continue
 
-            if line.startswith(('#')) or len(line) == 0:
-                continue
+                if line.startswith(("#")) or len(line) == 0:
+                    continue
 
-            # (If we got here it should be a valid line)
-            valid_lines += 1
-            yield feature_from_line(line, dialect=self.dialect)
+                # (If we got here it should be a valid line)
+                valid_lines += 1
+                yield feature_from_line(line, dialect=self.dialect)
 
 
 class _UrlIterator(_FileIterator):
     """
     Subclass for iterating over features provided as a URL
     """
+
+    @contextmanager
     def open_function(self, data):
         response = urlopen(data)
 
@@ -156,8 +167,9 @@ class _UrlIterator(_FileIterator):
         # http://stackoverflow.com/a/17537107
         # https://rationalpie.wordpress.com/2010/06/02/\
         #               python-streaming-gzip-decompression/
-        if data.endswith('.gz'):
+        if data.endswith(".gz"):
             import zlib
+
             d = zlib.decompressobj(16 + zlib.MAX_WBITS)
             READ_BLOCK_SIZE = 1024
 
@@ -168,21 +180,43 @@ class _UrlIterator(_FileIterator):
                     if not data:
                         break
                     data = "".join((last_line, d.decompress(data).decode()))
-                    lines = data.split('\n')
+                    lines = data.split("\n")
                     last_line = lines.pop()
                     for line in lines:
-                        yield line + '\n'
+                        yield line + "\n"
                 yield last_line
-            return _iter()
 
         else:
-            return response
+
+            def _iter():
+                for line in response.readlines():
+                    if not line:
+                        break
+                    yield line.decode() + "\n"
+
+        try:
+            yield _iter()
+        finally:
+            response.close()
 
 
 class _FeatureIterator(_BaseIterator):
     """
     Subclass for iterating over features that are already in an iterator
     """
+
+    def peek(self, n):
+        initial = []
+        for i, feature in enumerate(self.data):
+            initial.append(feature)
+            if i == n:
+                break
+
+        # If self.data is generator-like, we need to patch it back together.
+        if hasattr(self.data, "__next__"):
+            self.data = itertools.chain(initial, self.data)
+        return initial
+
     def _custom_iter(self):
         for i, feature in enumerate(self.data):
             self.current_item = feature
@@ -190,24 +224,6 @@ class _FeatureIterator(_BaseIterator):
             yield feature
 
 
-class _StringIterator(_FileIterator):
-    """
-    Subclass for iterating over features provided as a string (e.g., from
-    file.read())
-    """
-    def _custom_iter(self):
-        self.tmp = tempfile.NamedTemporaryFile(delete=False)
-        data = dedent(self.data)
-        if isinstance(data, six.text_type):
-            data = data.encode('utf-8')
-        self.tmp.write(data)
-        self.tmp.close()
-        self.data = self.tmp.name
-        for feature in super(_StringIterator, self)._custom_iter():
-            yield feature
-        os.unlink(self.tmp.name)
-
-
 def is_url(url):
     """
     Check to see if a URL has a valid protocol.
@@ -221,13 +237,21 @@ def is_url(url):
     True if `url` has a valid protocol False otherwise.
     """
     try:
-        return urlparse.urlparse(url).scheme in urlparse.uses_netloc
+        return urlparse.urlparse(url).scheme in set(urlparse.uses_netloc).difference(
+            [""]
+        )
     except:
         return False
 
 
-def DataIterator(data, checklines=10, transform=None,
-                 force_dialect_check=False, from_string=False, **kwargs):
+def DataIterator(
+    data,
+    checklines=10,
+    transform=None,
+    force_dialect_check=False,
+    from_string=False,
+    **kwargs,
+):
     """
     Iterate over features, no matter how they are provided.
 
@@ -261,18 +285,37 @@ def DataIterator(data, checklines=10, transform=None,
         `checklines=0` but this is not enforced.
     """
 
-    _kwargs = dict(data=data, checklines=checklines, transform=transform,
-                   force_dialect_check=force_dialect_check, **kwargs)
+    if isinstance(data, _BaseIterator):
+        return data
+
+    _kwargs = dict(
+        data=data,
+        checklines=checklines,
+        transform=transform,
+        force_dialect_check=force_dialect_check,
+        **kwargs,
+    )
     if isinstance(data, six.string_types):
         if from_string:
-            return _StringIterator(**_kwargs)
+            tmp = tempfile.NamedTemporaryFile(delete=False)
+            data = dedent(data)
+            if isinstance(data, six.text_type):
+                data = data.encode("utf-8")
+            tmp.write(data)
+            tmp.close()
+            _kwargs["data"] = tmp.name
+            return _FileIterator(**_kwargs)
         else:
             if os.path.exists(data):
                 return _FileIterator(**_kwargs)
             elif is_url(data):
                 return _UrlIterator(**_kwargs)
+            else:
+                raise ValueError(
+                    f"{data} cannot be found and does not " "appear to be a URL"
+                )
     elif isinstance(data, FeatureDB):
-        _kwargs['data'] = data.all_features()
+        _kwargs["data"] = data.all_features()
         return _FeatureIterator(**_kwargs)
 
     else:
